@@ -50,7 +50,6 @@
 
 
 typedef enum {
-	GTH_FILE_LIST_OP_TYPE_DELETE,
 	GTH_FILE_LIST_OP_TYPE_RENAME,
 	GTH_FILE_LIST_OP_TYPE_UPDATE_COMMENT,
 	GTH_FILE_LIST_OP_TYPE_UPDATE_THUMB,
@@ -60,7 +59,8 @@ typedef enum {
 	GTH_FILE_LIST_OP_TYPE_SET_LIST,
 	GTH_FILE_LIST_OP_TYPE_DELETE_LIST,
 	GTH_FILE_LIST_OP_TYPE_EMPTY_LIST,
-	GTH_FILE_LIST_OP_TYPE_ENABLE_THUMBS
+	GTH_FILE_LIST_OP_TYPE_ENABLE_THUMBS,
+	GTH_FILE_LIST_OP_TYPE_SET_FILTER
 } GthFileListOpType;
 
 
@@ -70,7 +70,7 @@ typedef struct {
 		GList *uri_list;
 		char  *uri;
 		int    ival;
-	};
+	} file;
 } GthFileListOp;
 
 
@@ -82,6 +82,7 @@ struct _GthFileListPrivateData {
 
 	GthFileView   *view;                /* The view that contains the
 			 	             * file list. */
+	GthFilter     *filter;
 
 	gboolean       show_dot_files;      /* Whether to show files that starts
 					     * with a dot (hidden files).*/
@@ -97,6 +98,7 @@ struct _GthFileListPrivateData {
 	int            thumb_pos;           /* The position of the item we are
 					     * genereting a thumbnail. */
 	guint          scroll_timer;
+	gboolean       update_thumb_in_image_list;
 
 	/**/
 
@@ -140,16 +142,15 @@ static void
 gth_file_list_op_free (GthFileListOp *op)
 {
 	switch (op->type) {
-	case GTH_FILE_LIST_OP_TYPE_DELETE:
 	case GTH_FILE_LIST_OP_TYPE_UPDATE_COMMENT:
 	case GTH_FILE_LIST_OP_TYPE_UPDATE_THUMB:
-		g_free (op->uri);
+		g_free (op->file.uri);
 		break;
 	case GTH_FILE_LIST_OP_TYPE_RENAME:
 	case GTH_FILE_LIST_OP_TYPE_UPDATE_THUMB_LIST:
 	case GTH_FILE_LIST_OP_TYPE_ADD_LIST:
 	case GTH_FILE_LIST_OP_TYPE_SET_LIST:
-		path_list_free (op->uri_list);
+		path_list_free (op->file.uri_list);
 		break;
 	default:
 		break;
@@ -167,12 +168,38 @@ gth_file_list_clear_queue (GthFileList *file_list)
 }
 
 
+static void
+gth_file_list_remove_op (GthFileList       *file_list,
+			 GthFileListOpType  op_type)
+{
+	GList *scan;
+
+	scan = file_list->priv->queue;
+	while (scan != NULL) {
+		GthFileListOp *op = scan->data;
+
+		if (op->type != op_type) {
+			scan = scan->next;
+			continue;
+		}
+
+		file_list->priv->queue =  g_list_remove_link (file_list->priv->queue, scan);
+		gth_file_list_op_free (op);
+		g_list_free (scan);
+
+		scan = file_list->priv->queue;
+	}
+}
+
+
 static void
 gth_file_list_queue_op (GthFileList   *file_list,
 			GthFileListOp *op)
 {
 	if (op->type == GTH_FILE_LIST_OP_TYPE_SET_LIST)
 		gth_file_list_clear_queue (file_list);
+	if (op->type == GTH_FILE_LIST_OP_TYPE_SET_FILTER)
+		gth_file_list_remove_op (file_list, GTH_FILE_LIST_OP_TYPE_SET_FILTER);
 	file_list->priv->queue = g_list_append (file_list->priv->queue, op);
 	if (! file_list->busy)
 		gth_file_list_exec_next_op (file_list);
@@ -187,7 +214,7 @@ gth_file_list_queue_op_with_uri (GthFileList       *file_list,
 	GthFileListOp *op;
 
 	op = gth_file_list_op_new (op_type);
-	op->uri = g_strdup (uri);
+	op->file.uri = g_strdup (uri);
 
 	gth_file_list_queue_op (file_list, op);
 }
@@ -201,7 +228,7 @@ gth_file_list_queue_op_with_list (GthFileList       *file_list,
 	GthFileListOp *op;
 
 	op = gth_file_list_op_new (op_type);
-	op->uri_list = path_list_dup (uri_list);
+	op->file.uri_list = path_list_dup (uri_list);
 
 	gth_file_list_queue_op (file_list, op);
 }
@@ -229,7 +256,7 @@ gth_file_list_queue_op_with_int (GthFileList       *file_list,
 	GthFileListOp *op;
 
 	op = gth_file_list_op_new (op_type);
-	op->ival = value;
+	op->file.ival = value;
 	gth_file_list_queue_op (file_list, op);
 }
 
@@ -266,7 +293,7 @@ gth_file_list_update_current_thumb (GthFileList *file_list)
 	gboolean  error = TRUE;
 	char     *path;
 
-	if (file_list->priv->stop_it) {
+	if (file_list->priv->stop_it || (file_list->priv->queue != NULL)) {
 		g_idle_add (update_thumbs_stopped, file_list);
 		return;
 	}
@@ -284,7 +311,7 @@ gth_file_list_update_current_thumb (GthFileList *file_list)
 
 		if (result == GNOME_VFS_OK) {
 			if (resolved_path != NULL) {
-				thumb_loader_set_path (file_list->priv->thumb_loader, resolved_path);
+				thumb_loader_set_path (file_list->priv->thumb_loader, resolved_path, file_list->priv->thumb_fd->mime_type);
 				thumb_loader_start (file_list->priv->thumb_loader);
 				error = FALSE;
 			}
@@ -322,7 +349,7 @@ gth_file_list_update_next_thumb (GthFileList *file_list)
 	GList    *list, *scan;
 	int       new_pos = -1;
 
-	if (file_list->priv->stop_it) {
+	if (file_list->priv->stop_it || (file_list->priv->queue != NULL)) {
 		g_idle_add (update_thumbs_stopped, file_list);
 		return;
 	}
@@ -333,7 +360,6 @@ gth_file_list_update_next_thumb (GthFileList *file_list)
 	last_pos = gth_file_view_get_last_visible (file_list->view);
 
 	pos = first_pos;
-
 	if ((pos == -1) || (last_pos < first_pos)) {
 		gth_file_list_done (file_list);
 		return;
@@ -341,9 +367,15 @@ gth_file_list_update_next_thumb (GthFileList *file_list)
 
 	list = gth_file_view_get_list (file_list->view);
 	scan = g_list_nth (list, pos);
+	if (scan == NULL) {
+		gth_file_list_done (file_list);
+		return;
+	}
+
+	/* Find a not loaded thumbnail among the visible images. */
 	while (pos <= last_pos) {
 		fd = scan->data;
-		if (! fd->thumb && ! fd->error) {
+		if (! fd->thumb_loaded && ! fd->error) {
 			new_pos = pos;
 			break;
 		} else {
@@ -351,6 +383,41 @@ gth_file_list_update_next_thumb (GthFileList *file_list)
 			scan = scan->next;
 		}
 	}
+
+	/* Find a not created thumbnail among the not-visible images. */
+
+	/* start from after the last visible image... */
+
+	if (new_pos == -1) {
+		pos = last_pos + 1;
+		scan = g_list_nth (list, pos);
+	        while (scan) {
+        	        fd = scan->data;
+                	if (! fd->thumb_created && ! fd->error) {
+                        	new_pos = pos;
+	                        break;
+        		}
+        		pos++;
+        		scan = scan->next;
+		}
+	}
+
+	/* ...continue from the beginning */
+
+	if (new_pos == -1) {
+		pos = 0;
+		scan = list;
+	        while (scan && (pos < first_pos)) {
+        	        fd = scan->data;
+                	if (! fd->thumb_created && ! fd->error) {
+                        	new_pos = pos;
+	                        break;
+        		}
+        		pos++;
+        		scan = scan->next;
+		}
+	}
+
 	g_list_free (list);
 
 	if (new_pos == -1) {
@@ -360,6 +427,7 @@ gth_file_list_update_next_thumb (GthFileList *file_list)
 
 	g_assert (fd != NULL);
 
+	file_list->priv->update_thumb_in_image_list = (new_pos >= first_pos) && (new_pos <= last_pos);
 	file_list->priv->thumb_pos = new_pos;
 	file_list->priv->thumbs_num++;
 
@@ -395,7 +463,8 @@ gth_file_list_update_thumbs (GthFileList *file_list)
 
 	for (scan = file_list->list; scan; scan = scan->next) {
 		FileData *fd = scan->data;
-		fd->thumb = FALSE;
+		fd->thumb_loaded = FALSE;
+		fd->thumb_created = FALSE;
 		fd->error = FALSE;
 	}
 
@@ -435,12 +504,32 @@ gth_file_list_enable_thumbs (GthFileList *file_list,
 
 
 static void
-update_thumb_in_clist (GthFileList *file_list)
+load_thumb_error_cb (ThumbLoader *tl,
+		     gpointer     data)
+{
+	GthFileList *file_list = data;
+
+	if (file_list->priv->stop_it || (file_list->priv->queue != NULL)) {
+		g_idle_add (update_thumbs_stopped, file_list);
+		return;
+	}
+
+	gth_file_view_set_unknown_pixbuf (file_list->view,  file_list->priv->thumb_pos);
+
+	file_list->priv->thumb_fd->error = TRUE;
+	file_list->priv->thumb_fd->thumb_loaded = FALSE;
+	file_list->priv->thumb_fd->thumb_created = FALSE;
+
+	gth_file_list_update_next_thumb (file_list);
+}
+
+
+static void
+update_thumb_in_image_list (GthFileList *file_list)
 {
 	GdkPixbuf *pixbuf;
 
 	pixbuf = thumb_loader_get_pixbuf (file_list->priv->thumb_loader);
-
 	if (pixbuf == NULL)
 		return;
 
@@ -450,40 +539,23 @@ update_thumb_in_clist (GthFileList *file_list)
 }
 
 
-static void
-load_thumb_error_cb (ThumbLoader *tl,
-		     gpointer     data)
-{
-	GthFileList *file_list = data;
-
-	if (file_list->priv->stop_it) {
-		g_idle_add (update_thumbs_stopped, file_list);
-		return;
-	}
-
-	gth_file_view_set_unknown_pixbuf (file_list->view,  file_list->priv->thumb_pos);
-
-	file_list->priv->thumb_fd->error = TRUE;
-	file_list->priv->thumb_fd->thumb = FALSE;
-
-	gth_file_list_update_next_thumb (file_list);
-}
-
-
 static void
 load_thumb_done_cb (ThumbLoader *tl,
 		    gpointer     data)
 {
 	GthFileList *file_list = data;
 
-	if (file_list->priv->stop_it) {
+	if (file_list->priv->stop_it || (file_list->priv->queue != NULL)) {
 		g_idle_add (update_thumbs_stopped, file_list);
 		return;
 	}
 
-	update_thumb_in_clist (file_list);
+	if (file_list->priv->update_thumb_in_image_list) {
+		update_thumb_in_image_list (file_list);
+		file_list->priv->thumb_fd->thumb_loaded = TRUE;
+	}
 	file_list->priv->thumb_fd->error = FALSE;
-	file_list->priv->thumb_fd->thumb = TRUE;
+	file_list->priv->thumb_fd->thumb_created = TRUE;
 
 	gth_file_list_update_next_thumb (file_list);
 }
@@ -517,6 +589,8 @@ gth_file_list_finalize (GObject *object)
 
 	gth_file_list_free_list (file_list);
 	g_object_unref (file_list->priv->thumb_loader);
+	if (file_list->priv->filter != NULL)
+		g_object_unref (file_list->priv->filter);
 
 	g_free (file_list->priv);
 
@@ -627,6 +701,7 @@ gth_file_list_init (GthFileList *file_list)
 	file_list->busy                 = FALSE;
 	file_list->priv->stop_it        = FALSE;
 	file_list->priv->loading_thumbs = FALSE;
+	file_list->priv->filter         = NULL;
 
 	g_signal_connect (G_OBJECT (file_list->priv->thumb_loader),
 			  "thumb_done",
@@ -739,6 +814,7 @@ typedef struct {
 	GList       *new_list;
 	GList       *filtered;
 	GList       *uri_list;
+	GHashTable  *mime_types;
 	guint        timeout_id;
 	gboolean     doing_thumbs;
 	gboolean     destroyed;
@@ -756,6 +832,7 @@ get_file_info_data_new (GthFileList *file_list)
 	file_list->priv->new_list = NULL;
 	data->filtered = NULL;
 	data->uri_list = NULL;
+	data->mime_types = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);
 	data->timeout_id = 0;
 	data->destroyed = FALSE;
 
@@ -769,6 +846,8 @@ get_file_info_data_free (GetFileInfoData *data)
 	if (data == NULL)
 		return;
 
+	g_hash_table_destroy (data->mime_types);
+
 	if (data->uri_list != NULL) {
 		g_list_foreach (data->uri_list,
 				(GFunc) gnome_vfs_uri_unref,
@@ -843,9 +922,9 @@ add_list_in_chunks (gpointer callback_data)
 	}
 
 	if (gfi_data->filtered == NULL) {
-		add_list_done (file_list);
 		gfi_data->destroyed = TRUE;
 		get_file_info_data_free (gfi_data);
+		add_list_done (file_list);
 		return FALSE;
 	}
 
@@ -858,15 +937,13 @@ add_list_in_chunks (gpointer callback_data)
 
 	for (i = 0, scan = gfi_data->filtered; (i < ADD_LIST_CHUNK_SIZE) && scan; i++, scan = scan->next) {
 		FileData *fd = scan->data;
-		int       pos;
 
 		file_data_update_comment (fd);
-
-		pos = gth_file_view_append_with_data (file_list->view,
-						      NULL,
-						      fd->utf8_name,
-						      fd->comment,
-						      fd);
+		gth_file_view_append_with_data (file_list->view,
+				      		NULL,
+						fd->display_name,
+						fd->comment,
+						fd);
 	}
 
 	gth_file_view_thaw (file_list->view);
@@ -920,7 +997,6 @@ set_list__get_file_info_done_cb (GnomeVFSAsyncHandle *handle,
 	g_signal_emit (G_OBJECT (file_list), gth_file_list_signals[IDLE], 0);
 
 	if (file_list->priv->stop_it) {
-		/*gnome_vfs_async_cancel (handle);*/
 		gfi_data->destroyed = TRUE;
 		g_idle_add (set_list_stopped, gfi_data);
 		return;
@@ -928,20 +1004,19 @@ set_list__get_file_info_done_cb (GnomeVFSAsyncHandle *handle,
 
 	for (scan = results; scan; scan = scan->next) {
 		GnomeVFSGetFileInfoResult *info_result = scan->data;
-		char                      *full_path;
-		char                      *escaped;
+		char                      *uri;
 		FileData                  *fd;
 
 		if ((info_result->result != GNOME_VFS_OK) || (info_result->uri == NULL))
 			continue;
 
-		escaped = gnome_vfs_uri_to_string (info_result->uri, GNOME_VFS_URI_HIDE_NONE);
-		full_path = gnome_vfs_unescape_string (escaped, "/");
-		fd = file_data_new (full_path, info_result->file_info);
+		uri = gnome_vfs_uri_to_string (info_result->uri, GNOME_VFS_URI_HIDE_NONE);
+
+		fd = file_data_new (uri, info_result->file_info);
+		fd->mime_type = g_hash_table_lookup (gfi_data->mime_types, uri);
 		gfi_data->filtered = g_list_prepend (gfi_data->filtered, fd);
 
-		g_free (full_path);
-		g_free (escaped);
+		g_free (uri);
 	}
 
 	add_list_in_chunks (gfi_data);
@@ -972,27 +1047,37 @@ load_new_list (GthFileList *file_list)
 	for (scan = gfi_data->new_list; scan; scan = scan->next) {
 		char        *full_path = scan->data;
 		const char  *name_only = file_name_from_path (full_path);
+		const char  *mime_type;
 		GnomeVFSURI *uri;
-		int          pos;
+		FileData    *fd;
+
+		mime_type = get_file_mime_type (full_path, fast_file_type);
+		if (mime_type == NULL)
+			continue;
 
 		if ((! file_list->priv->show_dot_files && file_is_hidden (name_only))
-		    || ! file_is_image (full_path, fast_file_type))
+		    || ! (mime_type_is_image (mime_type) || mime_type_is_video (mime_type)))
 			continue;
 
 		/* if the image is already present invalidate the thumbnail
 		 * instead of adding the image twice. */
 
-		pos = gth_file_list_pos_from_path (file_list, full_path);
-		if (pos != -1) {
-			FileData *fd = gth_file_view_get_image_data (file_list->view, pos);
+		fd = gth_file_list_filedata_from_path (file_list, full_path);
+		if (fd != NULL) {
+			fd->mime_type = mime_type;
 			fd->error = FALSE;
-			fd->thumb = FALSE;
+			fd->thumb_loaded = FALSE;
+			fd->thumb_created = FALSE;
 			continue;
 		}
 
 		uri = new_uri_from_path (full_path);
-		if (uri != NULL)
+		if (uri != NULL) {
 			gfi_data->uri_list = g_list_prepend (gfi_data->uri_list, uri);
+			g_hash_table_insert (gfi_data->mime_types,
+					     full_path,
+					     (gpointer) mime_type);
+		}
 	}
 
 	path_list_free (gfi_data->new_list);
@@ -1012,6 +1097,9 @@ void
 gfl_set_list (GthFileList *file_list,
 	      GList       *new_list)
 {
+	if (file_list->priv->filter != NULL)
+		gth_filter_reset (file_list->priv->filter);
+
 	gth_file_view_clear (file_list->view);
 	gth_file_list_free_list (file_list);
 
@@ -1163,6 +1251,32 @@ gth_file_list_pos_from_path (GthFileList *file_list,
 }
 
 
+FileData *
+gth_file_list_filedata_from_path (GthFileList *file_list,
+			          const char  *path)
+{
+	FileData *result = NULL;
+	GList    *list, *scan;
+
+	g_return_val_if_fail (file_list != NULL, NULL);
+
+	if (path == NULL)
+		return NULL;
+
+	list = gth_file_view_get_list (file_list->view);
+	for (scan = list; scan; scan = scan->next) {
+		FileData *fd = scan->data;
+		if (same_uri (fd->path, path)) {
+			result = fd;
+			break;
+		}
+	}
+	g_list_free (list);
+
+	return result;
+}
+
+
 GList *
 gth_file_list_get_all (GthFileList *file_list)
 {
@@ -1403,31 +1517,13 @@ static void
 gfl_delete (GthFileList *file_list,
             const char  *uri)
 {
-	int pos;
+	FileData *fd;
 
-	pos = gth_file_list_pos_from_path (file_list, uri);
-	if ((pos >= 0) && (pos < gth_file_view_get_images (file_list->view))) {
-		FileData *fd;
+	fd = gth_file_list_filedata_from_path (file_list, uri);
+	if (fd == NULL)
+		return;
 
-		fd = gth_file_view_get_image_data (file_list->view, pos);
-		g_return_if_fail (fd != NULL);
-
-		file_data_unref (fd);
-		file_list->list = g_list_remove (file_list->list, fd);
-		file_data_unref (fd);
-
-		gth_file_view_remove (file_list->view, pos);
-	}
-}
-
-
-void
-gth_file_list_delete (GthFileList *file_list,
-		      const char  *uri)
-{
-	gth_file_list_queue_op_with_uri (file_list,
-					 GTH_FILE_LIST_OP_TYPE_DELETE,
-					 uri);
+	gth_file_view_remove (file_list->view, fd);
 }
 
 
@@ -1441,6 +1537,8 @@ gfl_delete_list (GthFileList *file_list,
 	for (scan = uri_list; scan; scan = scan->next)
 		gfl_delete (file_list, scan->data);
 	gth_file_view_thaw (file_list->view);
+
+	g_signal_emit (file_list, gth_file_list_signals[DONE], 0);
 }
 
 
@@ -1454,31 +1552,41 @@ gth_file_list_delete_list (GthFileList *file_list,
 }
 
 
+void
+gth_file_list_delete (GthFileList *file_list,
+		      const char  *uri)
+{
+	gth_file_list_delete_list (file_list, g_list_append (NULL, (gpointer)uri));
+}
+
+
 static void
 gfl_rename (GthFileList *file_list,
 	    const char  *from_uri,
 	    const char  *to_uri)
 {
-	int pos;
+	FileData *fd;
+	int       pos;
+
+	fd = gth_file_list_filedata_from_path (file_list, from_uri);
+	if (fd == NULL)
+		return;
+
+	/* update the FileData structure. */
+
+	file_data_set_path (fd, to_uri);
+
+	/* Set the new name. */
 
 	pos = gth_file_list_pos_from_path (file_list, from_uri);
-	if ((pos >= 0) && (pos < gth_file_view_get_images (file_list->view))) {
-		FileData *fd;
-
-		/* update the FileData structure. */
-
-		fd = gth_file_view_get_image_data (file_list->view, pos);
-		file_data_set_path (fd, to_uri);
-
-		/* Set the new name. */
-
-		gth_file_view_set_image_text (file_list->view, pos, fd->utf8_name);
-		file_data_unref (fd);
-
+	if (pos != -1) {
+		gth_file_view_set_image_text (file_list->view, pos, fd->display_name);
 		gth_file_view_sorted (file_list->view,
 				      file_list->priv->sort_method,
 				      file_list->priv->sort_type);
 	}
+
+	file_data_unref (fd);
 }
 
 
@@ -1531,20 +1639,23 @@ static void
 gfl_update_thumb (GthFileList *file_list,
 		  const char  *uri)
 {
-	int pos;
+	FileData *fd;
+	int       pos;
+
+	fd = gth_file_list_filedata_from_path (file_list, uri);
+	if (fd == NULL)
+		return;
+
+	file_data_update (fd);
 
 	pos = gth_file_list_pos_from_path (file_list, uri);
-	if ((pos >= 0) && (pos < gth_file_view_get_images (file_list->view))) {
-		FileData *fd;
+	if (pos == -1)
+		return;
 
-		fd = gth_file_view_get_image_data (file_list->view, pos);
-		file_data_update (fd);
-
-		file_list->priv->thumb_pos = pos;
-		if (file_list->priv->thumb_fd != NULL)
-			file_data_unref (file_list->priv->thumb_fd);
-		file_list->priv->thumb_fd = fd;
-	}
+	file_list->priv->thumb_pos = pos;
+	if (file_list->priv->thumb_fd != NULL)
+		file_data_unref (file_list->priv->thumb_fd);
+	file_list->priv->thumb_fd = fd;
 }
 
 
@@ -1637,6 +1748,52 @@ gth_file_list_set_thumbs_size (GthFileList *file_list,
 }
 
 
+static gboolean
+gfl_visible_func (gpointer callback_data,
+    		  gpointer image_data)
+{
+	GthFileList *file_list = callback_data;
+	FileData    *fdata = image_data;
+
+	if (file_list->priv->filter == NULL)
+		return TRUE;
+	if (fdata == NULL)
+		return FALSE;
+
+	return gth_filter_match (file_list->priv->filter, fdata);
+}
+
+
+static void
+gfl_set_filter (GthFileList *file_list)
+{
+	if (file_list->priv->filter == NULL)
+		gth_file_view_set_visible_func (file_list->view, NULL, NULL);
+	else
+		gth_file_view_set_visible_func (file_list->view,
+                	                        gfl_visible_func,
+                        	                file_list);
+	g_signal_emit (file_list, gth_file_list_signals[DONE], 0);
+}
+
+
+void
+gth_file_list_set_filter (GthFileList *file_list,
+			  GthFilter   *filter)
+{
+	if (file_list->priv->filter == filter)
+		return;
+
+	if (file_list->priv->filter != NULL)
+		g_object_unref (file_list->priv->filter);
+	file_list->priv->filter = filter;
+	if (file_list->priv->filter != NULL)
+		g_object_ref (file_list->priv->filter);
+
+	gth_file_list_queue_op (file_list, gth_file_list_op_new (GTH_FILE_LIST_OP_TYPE_SET_FILTER));
+}
+
+
 static void
 gth_file_list_exec_next_op (GthFileList *file_list)
 {
@@ -1656,34 +1813,31 @@ gth_file_list_exec_next_op (GthFileList *file_list)
 	g_list_free (first);
 
 	switch (op->type) {
-	case GTH_FILE_LIST_OP_TYPE_DELETE:
-		gfl_delete (file_list, op->uri);
-		break;
 	case GTH_FILE_LIST_OP_TYPE_UPDATE_COMMENT:
-		gfl_update_comment (file_list, op->uri);
+		gfl_update_comment (file_list, op->file.uri);
 		break;
 	case GTH_FILE_LIST_OP_TYPE_UPDATE_THUMB:
-		gfl_update_thumb (file_list, op->uri);
+		gfl_update_thumb (file_list, op->file.uri);
 		break;
 	case GTH_FILE_LIST_OP_TYPE_RENAME:
-		gfl_rename (file_list, op->uri_list->data, op->uri_list->next->data);
+		gfl_rename (file_list, op->file.uri_list->data, op->file.uri_list->next->data);
 		break;
 	case GTH_FILE_LIST_OP_TYPE_UPDATE_THUMB_LIST:
-		gfl_update_thumb_list (file_list, op->uri_list);
+		gfl_update_thumb_list (file_list, op->file.uri_list);
 		break;
 	case GTH_FILE_LIST_OP_TYPE_ADD_LIST:
-		gfl_add_list (file_list, op->uri_list);
+		gfl_add_list (file_list, op->file.uri_list);
 		exec_next_op = FALSE;
 		break;
 	case GTH_FILE_LIST_OP_TYPE_SET_LIST:
-		gfl_set_list (file_list, op->uri_list);
+		gfl_set_list (file_list, op->file.uri_list);
 		exec_next_op = FALSE;
 		break;
 	case GTH_FILE_LIST_OP_TYPE_DELETE_LIST:
-		gfl_delete_list (file_list, op->uri_list);
+		gfl_delete_list (file_list, op->file.uri_list);
 		break;
 	case GTH_FILE_LIST_OP_TYPE_SET_THUMBS_SIZE:
-		gfl_set_thumbs_size (file_list, op->ival);
+		gfl_set_thumbs_size (file_list, op->file.ival);
 		exec_next_op = FALSE;
 		break;
 	case GTH_FILE_LIST_OP_TYPE_ENABLE_THUMBS:
@@ -1693,8 +1847,12 @@ gth_file_list_exec_next_op (GthFileList *file_list)
 	case GTH_FILE_LIST_OP_TYPE_EMPTY_LIST:
 		gfl_empty_list (file_list);
 		break;
+	case GTH_FILE_LIST_OP_TYPE_SET_FILTER:
+		gfl_set_filter (file_list);
+		exec_next_op = FALSE;
+		break;
 	}
-	op->uri_list = NULL;
+	op->file.uri_list = NULL;
 	gth_file_list_op_free (op);
 
 	if (exec_next_op)
