@@ -4,6 +4,7 @@
  *  GThumb
  *
  *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
+ *  Copyright (C) 2006-2007 Hubert Figuiere <hub@figuiere.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,6 +32,7 @@
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
+#include <libgnomevfs/gnome-vfs-mime.h>
 #include <gdk-pixbuf/gdk-pixbuf-animation.h>
 
 #include "gthumb-init.h"
@@ -45,7 +47,6 @@
 #include "jpeg-utils.h"
 #include "gthumb-marshal.h"
 
-
 #define DEFAULT_MAX_FILE_SIZE (4*1024*1024)
 
 #define THUMBNAIL_LARGE_SIZE	256
@@ -63,7 +64,7 @@ struct _ThumbLoaderPrivateData
 				    * image when done */
 
 	char *uri;
-	char *e_uri;
+	const char *mime_type;
 
 	gboolean use_cache : 1;
 	gboolean from_cache : 1;
@@ -129,7 +130,6 @@ thumb_loader_finalize (GObject *object)
 	g_object_unref (G_OBJECT (priv->il));
 
 	g_free (priv->uri);
-	g_free (priv->e_uri);
 
 	g_free (priv);
 	tl->priv = NULL;
@@ -192,7 +192,6 @@ thumb_loader_init (ThumbLoader *tl)
 
 	priv->thumb_factory = NULL;
 	priv->uri = NULL;
-	priv->e_uri = NULL;
 	priv->pixbuf = NULL;
 	priv->use_cache = TRUE;
 	priv->from_cache = FALSE;
@@ -231,53 +230,25 @@ thumb_loader_get_type ()
 
 static GdkPixbufAnimation*
 thumb_loader (const char  *path,
+              const char  *mime_type,
 	      GError     **error,
 	      gpointer     data)
 {
-	GdkPixbufAnimation *animation = NULL;
+	ThumbLoader *tl = data;
 
-	if (image_is_jpeg (path)) {
-		ThumbLoader *tl = data;
-		ThumbLoaderPrivateData *priv = tl->priv;
-		GdkPixbuf *pixbuf;
-
-		pixbuf = f_load_scaled_jpeg (path,
-					     priv->cache_max_w,
-					     priv->cache_max_h,
-					     NULL, NULL);
-
-		if (pixbuf != NULL) {
-			animation = gdk_pixbuf_non_anim_new (pixbuf);
-			g_object_unref (pixbuf);
-
-			if (animation == NULL)
-				debug (DEBUG_INFO, "ANIMATION == NULL\n");
-
-		} else
-			debug (DEBUG_INFO, "PIXBUF == NULL\n");
-
-	} else
-		{
-			if (image_is_gif (path))
-				animation = gdk_pixbuf_animation_new_from_file (path, error);
-			else {
-				GdkPixbuf *pixbuf;
-				pixbuf = gdk_pixbuf_new_from_file (path, error);
-				if (pixbuf != NULL) {
-					animation = gdk_pixbuf_non_anim_new (pixbuf);
-					g_object_unref (pixbuf);
-				}
-			}
-		}
-
-	return animation;
+	return gth_pixbuf_animation_new_from_uri (path,
+				       		  error,
+				       		  tl->priv->cache_max_w,
+				       		  tl->priv->cache_max_h,
+				       		  tl->priv->thumb_factory,
+				       		  mime_type);
 }
 
 
 GObject*
 thumb_loader_new (const char *path,
-		  int width,
-		  int height)
+		  int         width,
+		  int         height)
 {
 	ThumbLoaderPrivateData *priv;
 	ThumbLoader *tl;
@@ -300,11 +271,9 @@ thumb_loader_new (const char *path,
 	priv->max_h = height;
 
 	if (path != NULL)
-		thumb_loader_set_path (tl, path);
-	else {
+		thumb_loader_set_path (tl, path, NULL);
+	else
 		priv->uri = NULL;
-		priv->e_uri = NULL;
-	}
 
 	priv->il = IMAGE_LOADER (image_loader_new (path, FALSE));
 
@@ -364,24 +333,28 @@ thumb_loader_get_max_file_size (ThumbLoader      *tl)
 
 void
 thumb_loader_set_path (ThumbLoader *tl,
-		       const char  *path)
+		       const char  *path,
+		       const char  *mime_type)
 {
 	g_return_if_fail (tl != NULL);
 	g_return_if_fail (path != NULL);
 
 	g_free (tl->priv->uri);
-	g_free (tl->priv->e_uri);
 
 	tl->priv->uri = get_uri_from_path (path);
-	tl->priv->e_uri = gnome_vfs_escape_host_and_path_string (tl->priv->uri);
 
-	image_loader_set_path (tl->priv->il, remove_scheme_from_uri (tl->priv->uri));
+	if (mime_type == NULL)
+		mime_type = get_file_mime_type (path, eel_gconf_get_boolean (PREF_FAST_FILE_TYPE, TRUE));
+	tl->priv->mime_type = mime_type;
+
+	image_loader_set_path (tl->priv->il, tl->priv->uri, tl->priv->mime_type);
 }
 
 
 void
 thumb_loader_set_uri (ThumbLoader       *tl,
-		      const GnomeVFSURI *vfs_uri)
+		      const GnomeVFSURI *vfs_uri,
+		      const char        *mime_type)
 {
 	ThumbLoaderPrivateData *priv;
 
@@ -390,13 +363,11 @@ thumb_loader_set_uri (ThumbLoader       *tl,
 
 	priv = tl->priv;
 	g_free (priv->uri);
-	g_free (priv->e_uri);
 
+	priv->uri = gnome_vfs_uri_to_string (vfs_uri, GNOME_VFS_URI_HIDE_NONE);
+	tl->priv->mime_type = mime_type;
 
-	priv->e_uri = gnome_vfs_uri_to_string (vfs_uri, GNOME_VFS_URI_HIDE_NONE);
-	priv->uri = gnome_vfs_unescape_string (priv->e_uri, NULL);
-
-	image_loader_set_uri (priv->il, vfs_uri);
+	image_loader_set_uri (priv->il, vfs_uri, mime_type);
 }
 
 
@@ -432,7 +403,7 @@ thumb_loader_get_path (ThumbLoader *tl)
 	if (tl->priv->uri == NULL)
 		return NULL;
 
-	return g_strdup (remove_scheme_from_uri (tl->priv->uri));
+	return g_strdup (tl->priv->uri);
 }
 
 
@@ -454,12 +425,12 @@ thumb_loader_start__step2 (ThumbLoader *tl)
 		mtime = get_file_mtime (priv->uri);
 
 		cache_path = gnome_thumbnail_factory_lookup (priv->thumb_factory,
-							     priv->e_uri,
+							     priv->uri,
 							     mtime);
 
 		if ((cache_path == NULL)
 		    && gnome_thumbnail_factory_has_valid_failed_thumbnail (priv->thumb_factory,
-									   priv->e_uri,
+									   priv->uri,
 									   mtime)) {
 			g_signal_emit (G_OBJECT (tl),
 				       thumb_loader_signals[THUMB_ERROR],
@@ -470,12 +441,12 @@ thumb_loader_start__step2 (ThumbLoader *tl)
 
 	if (cache_path != NULL) {
 		priv->from_cache = TRUE;
-		image_loader_set_path (priv->il, cache_path);
+		image_loader_set_path (priv->il, cache_path, "image/png");
 		g_free (cache_path);
 
 	} else {
 		priv->from_cache = FALSE;
-		image_loader_set_path (priv->il, remove_scheme_from_uri (priv->uri));
+		image_loader_set_path (priv->il, priv->uri, priv->mime_type);
 
 		/* Check file dimensions. */
 
@@ -570,7 +541,7 @@ thumb_loader_save_to_cache (ThumbLoader *tl)
 
 	mtime = get_file_mtime (priv->uri);
 
-	cache_path = gnome_thumbnail_path_for_uri (priv->e_uri, GNOME_THUMBNAIL_SIZE_NORMAL);
+	cache_path = gnome_thumbnail_path_for_uri (priv->uri, GNOME_THUMBNAIL_SIZE_NORMAL);
 	cache_dir = remove_level_from_path (cache_path);
 	g_free (cache_path);
 
@@ -580,7 +551,7 @@ thumb_loader_save_to_cache (ThumbLoader *tl)
 	if (ensure_dir_exists (cache_dir, THUMBNAIL_DIR_PERMISSIONS))
 		gnome_thumbnail_factory_save_thumbnail (priv->thumb_factory,
 							priv->pixbuf,
-							priv->e_uri,
+							priv->uri,
 							mtime);
 
 	g_free (cache_dir);
@@ -608,7 +579,7 @@ thumb_loader_done_cb (ImageLoader *il,
 
 	if (pixbuf == NULL) {
 		gnome_thumbnail_factory_create_failed_thumbnail (priv->thumb_factory,
-								 priv->e_uri,
+								 priv->uri,
 								 get_file_mtime (priv->uri));
 		g_signal_emit (G_OBJECT (tl), thumb_loader_signals[THUMB_ERROR], 0);
 		return;
@@ -685,7 +656,7 @@ thumb_loader_error_cb (ImageLoader *il,
 		}
 
 		gnome_thumbnail_factory_create_failed_thumbnail (priv->thumb_factory,
-								 priv->e_uri,
+								 priv->uri,
 								 get_file_mtime (priv->uri));
 		g_signal_emit (G_OBJECT (tl), thumb_loader_signals[THUMB_ERROR], 0);
 
@@ -696,7 +667,7 @@ thumb_loader_error_cb (ImageLoader *il,
 	priv->from_cache = FALSE;
 	g_warning ("Thumbnail image in cache failed to load, trying to recreate.");
 
-	image_loader_set_path (priv->il, remove_scheme_from_uri (priv->uri));
+	image_loader_set_path (priv->il, priv->uri, priv->mime_type);
 	image_loader_start (priv->il);
 }
 
