@@ -23,17 +23,18 @@
 #include <config.h>
 #include <string.h>
 
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <libgnome/libgnome.h>
-#include <libgnomeui/gnome-dialog.h>
-#include <libgnomeui/gnome-dialog-util.h>
-#include <libgnomeui/gnome-dateedit.h>
+#include <libgnomeui/gnome-help.h>
 #include <glade/glade.h>
+
 #include "typedefs.h"
+#include "file-utils.h"
 #include "preferences.h"
 #include "main.h"
-#include "gthumb-window.h"
+#include "gth-utils.h"
+#include "gth-window.h"
 #include "gtk-utils.h"
 #include "gtkcellrendererthreestates.h"
 #include "gth-file-view.h"
@@ -69,7 +70,7 @@ typedef struct {
 	DoneFunc       done_func;
 	gpointer       done_data;
 
-	GThumbWindow  *window;
+	GthWindow     *window;
 	GtkWidget     *dialog;
 
 	GtkWidget     *c_ok_button;
@@ -106,7 +107,8 @@ destroy_cb (GtkWidget  *widget,
 	    DialogData *data)
 {
 	if (data->window != NULL)
-		data->window->categories_dlg = NULL;
+		gth_window_set_categories_dlg (data->window, NULL);
+
 	g_object_unref (data->gui);
 	free_dialog_data (data);
 	path_list_free (data->default_categories_list);
@@ -262,7 +264,6 @@ add_category_cb (GtkWidget  *widget,
 				    CATEGORY_COLUMN, new_category,
 				    -1);
 		update_category_entry (data);
-		gtk_widget_set_sensitive (data->c_ok_button, TRUE);
 	}
 
 	g_free (new_category);
@@ -283,7 +284,6 @@ remove_category_cb (GtkWidget *widget,
 
 	gtk_list_store_remove (data->keywords_list_model, &iter);
 	update_category_entry (data);
-	gtk_widget_set_sensitive (data->c_ok_button, TRUE);
 }
 
 
@@ -341,9 +341,10 @@ ok_clicked_cb (GtkWidget  *widget,
 
 	if (data->save_func) {
 		(*data->save_func) (data->file_list, data->save_data);
-		gtk_widget_set_sensitive (data->c_ok_button, FALSE);
-		if (data->window != NULL)
-			window_reload_image (data->window);
+		if (data->window != NULL) {
+			gth_window_update_current_image_metadata (data->window);
+			dlg_categories_close (data->dialog);
+		}
 	} else
 		gtk_widget_destroy (data->dialog);
 }
@@ -354,31 +355,7 @@ static void
 help_cb (GtkWidget  *widget, 
 	 DialogData *data)
 {
-	GError *err;
-
-	err = NULL;  
-	gnome_help_display ("gthumb", "comments", &err);
-	
-	if (err != NULL) {
-		GtkWidget *dialog;
-		
-		dialog = gtk_message_dialog_new (GTK_WINDOW (data->dialog),
-						 0,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
-						 _("Could not display help: %s"),
-						 err->message);
-		
-		g_signal_connect (G_OBJECT (dialog), "response",
-				  G_CALLBACK (gtk_widget_destroy),
-				  NULL);
-		
-		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-		
-		gtk_widget_show (dialog);
-		
-		g_error_free (err);
-	}
+	gthumb_display_help (GTK_WINDOW (data->dialog), "gthumb-categories");
 }
 
 
@@ -399,7 +376,6 @@ use_category_toggled (GtkCellRendererThreeStates *cell,
 	
 	gtk_tree_path_free (path);
 	update_category_entry (data);
-	gtk_widget_set_sensitive (data->c_ok_button, TRUE);
 }
 
 
@@ -477,12 +453,25 @@ name_column_sort_func (GtkTreeModel *model,
 }
 
 
+static gboolean
+keyword_equal_func (GtkTreeModel *model,
+		    gint          column, 
+		    const gchar  *key,
+		    GtkTreeIter  *iter,
+		    gpointer      search_data)
+{
+	char *cell;
+	gtk_tree_model_get (model, iter, column, &cell, -1);
+	return g_strcasecmp (key, cell) > 0;
+}
+    
+
 
 
 
 static GtkWidget*
 dlg_categories_common (GtkWindow     *parent,
-		       GThumbWindow  *window,
+		       GthWindow     *window,
 		       GList         *file_list,
 		       GList         *default_categories_list,
 		       GList        **add_categories_list,
@@ -538,11 +527,6 @@ dlg_categories_common (GtkWindow     *parent,
 	btn_cancel = glade_xml_get_widget (data->gui, "c_cancel_button");
 	btn_help = glade_xml_get_widget (data->gui, "c_help_button");
 
-	if (!modal) {
-		gtk_button_set_label (GTK_BUTTON (btn_ok), GTK_STOCK_SAVE);
-		gtk_button_set_label (GTK_BUTTON (btn_cancel), GTK_STOCK_CLOSE);
-	}
-
 	/* Set widgets data. */
 
 	g_object_set_data (G_OBJECT (data->dialog), "dialog_data", data);
@@ -557,6 +541,13 @@ dlg_categories_common (GtkWindow     *parent,
 	g_object_unref (data->keywords_list_model);
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (data->keywords_list_view), FALSE);
 
+	gtk_tree_view_set_search_column (GTK_TREE_VIEW (data->keywords_list_view), 
+					 CATEGORY_COLUMN);
+	gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (data->keywords_list_view),
+					     keyword_equal_func,
+					     NULL,
+					     NULL);
+
 	renderer = gtk_cell_renderer_three_states_new ();
 	g_signal_connect (G_OBJECT (renderer), 
 			  "toggled",
@@ -661,7 +652,7 @@ dlg_choose_categories (GtkWindow     *parent,
 
 
 typedef struct {
-	GThumbWindow  *window;
+	GthWindow     *window;
 	GList         *add_list;
 	GList         *remove_list;
 } DlgCategoriesData;
@@ -694,7 +685,7 @@ dlg_categories__save (GList    *file_list,
 		CommentData *cdata;
 		GList       *scan2;
 
-		cdata = comments_load_comment (filename);
+		cdata = comments_load_comment (filename, TRUE);
 		if (cdata == NULL)
 			cdata = comment_data_new ();
 		else 
@@ -721,14 +712,15 @@ dlg_categories__save (GList    *file_list,
 
 
 GtkWidget*
-dlg_categories_new (GThumbWindow *window)
+dlg_categories_new (GthWindow *window)
 {
-	GtkWidget         *parent_win = window->viewer;
+	GtkWidget         *current_dlg;
 	DlgCategoriesData *dcdata;
 
-	if (window->categories_dlg != NULL) {
-		gtk_window_present (GTK_WINDOW (window->categories_dlg));
-		return window->categories_dlg;
+	current_dlg = gth_window_get_categories_dlg (window);
+	if (current_dlg != NULL) {
+		gtk_window_present (GTK_WINDOW (current_dlg));
+		return current_dlg;
 	}
 
 	dcdata = g_new0 (DlgCategoriesData, 1);
@@ -736,10 +728,7 @@ dlg_categories_new (GThumbWindow *window)
 	dcdata->add_list = NULL;
 	dcdata->remove_list = NULL;
 
-	while (parent_win != NULL && !GTK_IS_WINDOW(parent_win))
-		parent_win = parent_win->parent;
-
-	return dlg_categories_common (GTK_WINDOW (parent_win),
+	return dlg_categories_common (GTK_WINDOW (window),
 				      window,
 				      NULL,
 				      NULL,
@@ -773,7 +762,7 @@ dlg_categories_update (GtkWidget *dlg)
 
 	if (data->window != NULL) {
 		free_dialog_data (data);
-		data->file_list = gth_file_view_get_file_list_selection (data->window->file_list->view);
+		data->file_list = gth_window_get_file_list_selection (data->window);
 	} else if (data->original_cdata != NULL) {
 		comment_data_free (data->original_cdata);
 		data->original_cdata = NULL;
@@ -781,7 +770,7 @@ dlg_categories_update (GtkWidget *dlg)
 
 	if (data->file_list != NULL) {
 		char *first_image = data->file_list->data;
-		data->original_cdata = cdata = comments_load_comment (first_image);
+		data->original_cdata = cdata = comments_load_comment (first_image, TRUE);
 	}
 	
 	if (cdata != NULL) {
@@ -792,7 +781,7 @@ dlg_categories_update (GtkWidget *dlg)
 			CommentData *scan_cdata;
 			int          i;
 
-			scan_cdata = comments_load_comment (scan->data);
+			scan_cdata = comments_load_comment (scan->data, TRUE);
 		
 			if (scan_cdata == NULL) {
 				comment_data_free_keywords (cdata);
@@ -824,7 +813,7 @@ dlg_categories_update (GtkWidget *dlg)
 		CommentData *scan_cdata;
 		int          j;
 
-		scan_cdata = comments_load_comment (scan->data);
+		scan_cdata = comments_load_comment (scan->data, TRUE);
 		
 		if (scan_cdata == NULL) 
 			continue;
@@ -903,7 +892,6 @@ dlg_categories_update (GtkWidget *dlg)
 
 	add_saved_categories (data);
 	update_category_entry (data);
-	gtk_widget_set_sensitive (data->c_ok_button, FALSE);
 }
 
 
