@@ -29,6 +29,7 @@
 #include <libgnomecanvas/libgnomecanvas.h>
 
 #include "glib-utils.h"
+#include "gth-utils.h"
 #include "gth-window.h"
 #include "gconf-utils.h"
 #include "preferences.h"
@@ -41,7 +42,6 @@
 #include "typedefs.h"
 #include "gth-nav-window.h"
 
-#define PREVIEW_SIZE   400
 #define GLADE_FILE     "gthumb_crop.glade"
 
 /**/
@@ -67,7 +67,7 @@ typedef struct {
 	GtkSpinButton *ratio_w_spinbutton;
 	GtkSpinButton *ratio_h_spinbutton;
 	GtkWidget     *custom_ratio_box;
-	GtkWidget     *ratio_swap_button;
+	GtkWidget     *invert_ratio_checkbutton;
 	GtkWidget     *mask_button;
 	GtkWidget     *undo_button;
 	GtkWidget     *redo_button;
@@ -132,6 +132,14 @@ save_cb (GtkWidget  *widget,
 }
 
 
+static void
+help_cb (GtkWidget  *widget,
+         DialogData *data)
+{
+	gthumb_display_help (GTK_WINDOW (data->dialog), "gthumb-crop");
+}
+
+
 static void
 selection_x_value_changed_cb (GtkSpinButton *spin,
 			      DialogData    *data)
@@ -273,6 +281,7 @@ ratio_optionmenu_changed_cb (GtkOptionMenu *optionmenu,
 	}
 
 	gtk_widget_set_sensitive (data->custom_ratio_box, idx == GTH_CROP_RATIO_CUSTOM);
+	gtk_widget_set_sensitive (data->invert_ratio_checkbutton, use_ratio);
 	set_spin_value (data, data->ratio_w_spinbutton, w);
 	set_spin_value (data, data->ratio_h_spinbutton, h);
 
@@ -286,32 +295,22 @@ static void
 ratio_value_changed_cb (GtkSpinButton *spin,
 			DialogData    *data)
 {
-	int w, h;
+	gboolean use_ratio;
+	int      w, h;
+	double   ratio;
+
+	use_ratio = gtk_option_menu_get_history (GTK_OPTION_MENU (data->ratio_optionmenu)) != GTH_CROP_RATIO_NONE;
 
 	w = gtk_spin_button_get_value_as_int (data->ratio_w_spinbutton);
 	h = gtk_spin_button_get_value_as_int (data->ratio_h_spinbutton);
 
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->invert_ratio_checkbutton)))
+		ratio = (double) h / w;
+	else
+		ratio = (double) w / h;
 	gth_image_selector_set_ratio (GTH_IMAGE_SELECTOR (data->image_selector),
-				      TRUE,
-				      (double) w / h);
-}
-
-
-static void
-ratio_swap_button_cb (GtkButton *button,
-		      DialogData *data)
-{
-	int w, h;
-
-	w = gtk_spin_button_get_value_as_int (data->ratio_w_spinbutton);
-	h = gtk_spin_button_get_value_as_int (data->ratio_h_spinbutton);
-
-	set_spin_value (data, data->ratio_w_spinbutton, h);
-	set_spin_value (data, data->ratio_h_spinbutton, w);
-
-	gth_image_selector_set_ratio (GTH_IMAGE_SELECTOR (data->image_selector),
-				      TRUE,
-				      (double) h / w);
+				      use_ratio,
+				      ratio);
 }
 
 
@@ -453,12 +452,14 @@ dlg_crop (GthWindow *window)
 	GtkWidget    *ok_button;
 	GtkWidget    *save_button;
 	GtkWidget    *cancel_button;
+	GtkWidget    *help_button;
 	GtkWidget    *menu, *item;
 	GtkWidget    *image;
 	GtkWidget    *crop_button, *zoom_in_button, *zoom_out_button, *zoom_100_button, *zoom_fit_button;
 	char         *label;
 	GthCropRatio  crop_ratio;
 	int           ratio_w, ratio_h;
+	GdkScreen    *screen;
 
 	data = g_new0 (DialogData, 1);
 	data->window = window;
@@ -485,7 +486,7 @@ dlg_crop (GthWindow *window)
 	data->ratio_w_spinbutton = (GtkSpinButton*) glade_xml_get_widget (data->gui, "ratio_w_spinbutton");
 	data->ratio_h_spinbutton = (GtkSpinButton*) glade_xml_get_widget (data->gui, "ratio_h_spinbutton");
 	data->custom_ratio_box = glade_xml_get_widget (data->gui, "custom_ratio_box");
-	data->ratio_swap_button = glade_xml_get_widget (data->gui, "ratio_swap_button");
+	data->invert_ratio_checkbutton = glade_xml_get_widget (data->gui, "invert_ratio_checkbutton");
 
 	crop_button = glade_xml_get_widget (data->gui, "crop_crop_button");
 	data->undo_button = glade_xml_get_widget (data->gui, "crop_undo_button");
@@ -499,12 +500,10 @@ dlg_crop (GthWindow *window)
 	ok_button = glade_xml_get_widget (data->gui, "crop_okbutton");
 	save_button = glade_xml_get_widget (data->gui, "crop_savebutton");
 	cancel_button = glade_xml_get_widget (data->gui, "crop_cancelbutton");
+	help_button = glade_xml_get_widget (data->gui, "crop_helpbutton");
 
 	data->image_selector = gth_image_selector_new (GTH_SELECTOR_TYPE_REGION, NULL);
 
-	image = glade_xml_get_widget (data->gui, "ratio_swap_image");
-	gtk_image_set_from_stock (GTK_IMAGE (image), GTHUMB_STOCK_SWAP, GTK_ICON_SIZE_MENU);
-
 	image = glade_xml_get_widget (data->gui, "crop_image");
 	gtk_image_set_from_stock (GTK_IMAGE (image), GTHUMB_STOCK_CROP, GTK_ICON_SIZE_MENU);
 
@@ -516,8 +515,6 @@ dlg_crop (GthWindow *window)
 	gth_image_selector_set_pixbuf (GTH_IMAGE_SELECTOR (data->image_selector), data->pixbuf);
 
 	data->nav_win = gth_nav_window_new (GTH_IVIEWER (data->image_selector));
-	gtk_widget_set_size_request (data->nav_win, PREVIEW_SIZE, PREVIEW_SIZE);
-
 	gtk_container_add (GTK_CONTAINER (data->nav_container), data->nav_win);
 
 	gtk_widget_show_all (data->nav_container);
@@ -558,7 +555,7 @@ dlg_crop (GthWindow *window)
 				     crop_ratio);
 	gtk_widget_set_sensitive (data->custom_ratio_box,
 				  (crop_ratio == GTH_CROP_RATIO_CUSTOM));
-
+        gtk_widget_set_sensitive (data->invert_ratio_checkbutton, (crop_ratio != GTH_CROP_RATIO_NONE));
 	gth_image_selector_set_ratio (GTH_IMAGE_SELECTOR (data->image_selector),
 				      (crop_ratio != GTH_CROP_RATIO_NONE),
 				      (double) ratio_w / ratio_h);
@@ -581,6 +578,10 @@ dlg_crop (GthWindow *window)
 			  "clicked",
 			  G_CALLBACK (save_cb),
 			  data);
+	g_signal_connect (G_OBJECT (help_button),
+			  "clicked",
+			  G_CALLBACK (help_cb),
+			  data);
 	g_signal_connect (G_OBJECT (data->crop_x_spinbutton),
 			  "value_changed",
 			  G_CALLBACK (selection_x_value_changed_cb),
@@ -617,9 +618,9 @@ dlg_crop (GthWindow *window)
 			  "value_changed",
 			  G_CALLBACK (ratio_value_changed_cb),
 			  data);
-	g_signal_connect (G_OBJECT (data->ratio_swap_button),
-			  "clicked",
-			  G_CALLBACK (ratio_swap_button_cb),
+	g_signal_connect (G_OBJECT (data->invert_ratio_checkbutton),
+			  "toggled",
+			  G_CALLBACK (ratio_value_changed_cb),
 			  data);
 
 	g_signal_connect (G_OBJECT (crop_button),
@@ -662,6 +663,12 @@ dlg_crop (GthWindow *window)
 	/* Run dialog. */
 
 	gtk_widget_realize (data->dialog);
+
+	screen = gdk_screen_get_default();
+	gtk_window_set_default_size (GTK_WINDOW (data->dialog),
+				     gdk_screen_get_width (screen) * 7 / 10,
+				     gdk_screen_get_height (screen) * 6 / 10);
+
 	gth_iviewer_zoom_to_fit (GTH_IVIEWER (data->image_selector));
 	gtk_widget_grab_focus (data->image_selector);
 
