@@ -23,10 +23,12 @@
 #ifndef DLG_JPEGTRAN_H
 #define DLG_JPEGTRAN_H
 
-#include "gthumb-window.h"
+#include "typedefs.h"
+#include "gth-window.h"
 
 
-void dlg_jpegtran (GThumbWindow *window); 
-
+void dlg_jpegtran                 (GthWindow    *window);
+void dlg_apply_jpegtran           (GthWindow    *window,
+				   GthTransform  transform);
 
 #endif /* DLG_JPEGTRAN_H */
