@@ -23,22 +23,20 @@
 #include <config.h>
 #include <gnome.h>
 #include <glade/glade.h>
-#include <libgnomeui/gnome-window-icon.h>
-#include <libgnomeui/gnome-ui-init.h>
+#include <gdk/gdkx.h>
 #include <libgnomevfs/gnome-vfs-init.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
-#include <libbonobo.h>
+#include <libgnomeui/gnome-authentication-manager.h>
 
 #include "catalog.h"
 #include "comments.h"
-#include "dir-list.h"
 #include "main.h"
 #include "file-utils.h"
 #include "gconf-utils.h"
 #include "gtk-utils.h"
-#include "gth-application.h"
 #include "gth-browser.h"
 #include "gth-browser-actions-callbacks.h"
+#include "gth-dir-list.h"
 #include "gth-image-list.h"
 #include "gth-viewer.h"
 #include "gthumb-init.h"
@@ -63,36 +61,53 @@ static char*icon_mime_name[ICON_NAMES] = { "gnome-fs-directory",
 					   "gnome-fs-home",
 					   "gnome-fs-desktop" };
 
-GthMonitor  *monitor = NULL;
-char       **file_urls, **dir_urls;
-int          n_file_urls, n_dir_urls;
-int          StartInFullscreen = FALSE;
-int          StartSlideshow = FALSE;
-int          ViewFirstImage = FALSE;
-int          HideSidebar = FALSE;
-gboolean     ExitAll = FALSE;
-char        *ImageToDisplay = NULL;
-gboolean     FirstStart = TRUE;
-gboolean     ImportPhotos = FALSE;
+#ifdef HAVE_GTKUNIQUE
+GtkUniqueApp  *gth_application = NULL;
+#endif /* HAVE_GTKUNIQUE */
 
-static gboolean        view_comline_catalog = FALSE;
-static gboolean        view_single_image = FALSE;
-static GdkPixbuf      *icon_pixbuf[ICON_NAMES] = { 0 };
-static GtkWidget      *first_window = NULL;
-static GnomeIconTheme *icon_theme = NULL;
-static BonoboObject   *gth_application = NULL;
+GthMonitor    *monitor = NULL;
+GList         *file_urls = NULL, *dir_urls = NULL;
+int            n_file_urls, n_dir_urls;
+int            StartInFullscreen = FALSE;
+int            StartSlideshow = FALSE;
+int            ViewFirstImage = FALSE;
+int            HideSidebar = FALSE;
+gboolean       ExitAll = FALSE;
+char          *ImageToDisplay = NULL;
+gboolean       FirstStart = TRUE;
+gboolean       ImportPhotos = FALSE;
+gboolean       UseViewer = FALSE;
 
-struct poptOption options[] = {
-	{ "fullscreen", '\0', POPT_ARG_NONE, &StartInFullscreen, 0,
+static gboolean         view_comline_catalog = FALSE;
+static gboolean         view_single_image = FALSE;
+static GdkPixbuf       *icon_pixbuf[ICON_NAMES] = { 0 };
+static GtkWidget       *first_window = NULL;
+static GtkIconTheme    *icon_theme = NULL;
+static GOptionContext  *context;
+static char           **remaining_args;
+
+static const GOptionEntry options[] = {
+	{ "fullscreen", 'f', 0, G_OPTION_ARG_NONE, &StartInFullscreen,
 	  N_("Start in fullscreen mode"),
 	  0 },
-	{ "slideshow", '\0', POPT_ARG_NONE, &StartSlideshow, 0,
+
+	{ "slideshow", 's', 0, G_OPTION_ARG_NONE, &StartSlideshow,
 	  N_("Automatically start a slideshow"),
 	  0 },
-	{ "import-photos", '\0', POPT_ARG_NONE, &ImportPhotos, 0,
+
+	{ "import-photos", '\0', 0, G_OPTION_ARG_NONE, &ImportPhotos,
 	  N_("Automatically import digital camera photos"),
 	  0 },
-	{ NULL, '\0', 0, NULL, 0 }
+
+	{ "viewer", '\0', 0, G_OPTION_ARG_NONE, &UseViewer,
+	  N_("Use the viewer mode to view single images"),
+	  0 },
+
+	{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, &remaining_args,
+          NULL,
+          NULL },
+
+	{ NULL }
 };
 
 
@@ -100,7 +115,7 @@ static void
 init_icon_pixbufs (void)
 {
 	int i;
-	for (i = 0; i < ICON_NAMES; i++) 
+	for (i = 0; i < ICON_NAMES; i++)
 		icon_pixbuf[i] = NULL;
 }
 
@@ -118,7 +133,7 @@ free_icon_pixbufs (void)
 
 
 static void
-theme_changed_cb (GnomeIconTheme *theme, 
+theme_changed_cb (GtkIconTheme *theme,
 		  gpointer        data)
 {
 	free_icon_pixbufs ();
@@ -130,7 +145,7 @@ static void
 create_default_categories_if_needed (void)
 {
 	Bookmarks *categories;
-	char      *default_categories[] = { N_("Holidays"), 
+	char      *default_categories[] = { N_("Holidays"),
 					    N_("Temporary"),
 					    N_("Screenshots"),
 					    N_("Science"),
@@ -157,22 +172,22 @@ create_default_categories_if_needed (void)
 	g_free (path);
 
 	categories = bookmarks_new (RC_CATEGORIES_FILE);
-	for (i = 0; default_categories[i] != NULL; i++) 
+	for (i = 0; default_categories[i] != NULL; i++)
 		bookmarks_add (categories, _(default_categories[i]), TRUE, TRUE);
 	bookmarks_write_to_disk (categories);
 	bookmarks_free (categories);
 }
 
 
-static void 
-convert_old_comment (char     *real_file, 
-		     char     *rc_file, 
+static void
+convert_old_comment (char     *real_file,
+		     char     *rc_file,
 		     gpointer  data)
 {
 	char *comment_file;
 	char *comment_dir;
 
-	comment_file = comments_get_comment_filename (real_file, TRUE, TRUE);
+	comment_file = comments_get_comment_filename (real_file, TRUE);
 	comment_dir = remove_level_from_path (comment_file);
 	ensure_dir_exists (comment_dir, 0755);
 
@@ -242,14 +257,14 @@ save_session (GnomeClient *client)
 					continue;
 				uri = get_uri_from_path (location);
 				break;
-			
+
 			case GTH_SIDEBAR_CATALOG_LIST:
 				location = gth_browser_get_current_catalog (browser);
 				if (location == NULL)
 					continue;
 				uri = get_uri_from_path (location);
 				break;
-				
+
 			default:
 				break;
 			}
@@ -303,7 +318,7 @@ client_save_yourself_cb (GnomeClient *client,
 
 	argv[0] = (char *) program_argv0;
 	argv[1] = NULL; /* "--debug-session"; */
-	
+
 	gnome_client_set_clone_command (client, 1, argv);
 	gnome_client_set_restart_command (client, 1, argv);
 
@@ -346,7 +361,7 @@ static gboolean
 session_is_restored (void)
 {
 	gboolean restored;
-	
+
 	if (! master_client)
 		return FALSE;
 
@@ -357,16 +372,14 @@ session_is_restored (void)
 
 
 /* Initialize application data. */
-static void 
-initialize_data (poptContext pctx)
+static void
+initialize_data (void)
 {
-	const char **argv;
-	int          argc;
-	char        *current_dir;
-	char        *path;
-	int          i;
+	char *current_dir;
+	char *path, *filename;
+	int   i = 0;
 
-	convert_to_new_comment_system (); 
+	convert_to_new_comment_system ();
 	create_default_categories_if_needed ();
 
 	eel_gconf_monitor_add ("/apps/gthumb");
@@ -375,8 +388,7 @@ initialize_data (poptContext pctx)
 
 	/* Icon theme */
 
-	icon_theme = gnome_icon_theme_new ();
-	gnome_icon_theme_set_allow_svg (icon_theme, TRUE);
+	icon_theme = gtk_icon_theme_get_default ();
 	g_signal_connect (icon_theme,
 			  "changed",
 			  G_CALLBACK (theme_changed_cb),
@@ -385,55 +397,32 @@ initialize_data (poptContext pctx)
 	/* Default windows icon */
 
 	init_icon_pixbufs ();
+	g_set_application_name (_("gThumb"));
 	gtk_window_set_default_icon_name ("gthumb");
-	gtk_window_set_default_icon_from_file (PIXMAPSDIR "gthumb.png", NULL);
 
 	/**/
 
 	init_session ("gthumb");
-	if (session_is_restored ()) 
+	if (session_is_restored ())
 		return;
 
 	/* Parse command line arguments. */
 
-	argv = poptGetArgs (pctx);
-
-	if (argv == NULL)
+	if (remaining_args == NULL) /* No arguments specified. */
 		return;
 
-	for (argc = 0; argv[argc] != NULL; argc++) 
-		;
-
-	if (argc == 0)
-		return;
-
-	file_urls = g_new0 (char*, argc);
-	dir_urls = g_new0 (char*, argc);
-
-	n_file_urls = 0;
-	n_dir_urls = 0;
-
 	current_dir = g_get_current_dir ();
-	for (i = 0; i < argc; i++) {
-		char     *tmp1, *tmp2;
+	while ((filename = remaining_args[i++]) != NULL) {
+		char     *tmp1 = NULL;
 		gboolean  is_dir;
 
-		if (! g_path_is_absolute (argv[i]))
-			tmp1 = g_strconcat (current_dir, "/", argv[i], NULL);
+		if (uri_has_scheme (filename) || g_path_is_absolute (filename))
+			tmp1 = gnome_vfs_make_uri_from_shell_arg (filename);
 		else
-			tmp1 = g_strdup (argv[i]);
+			tmp1 = g_strconcat (current_dir, "/", filename, NULL);
 
-		tmp2= remove_special_dirs_from_path (tmp1);
-
-		if (! g_path_is_absolute (tmp2))
-			path = g_strconcat (current_dir, "/", tmp2, NULL);
-		else {
-			path = tmp2;
-			tmp2 = NULL;
-		}
-		
+		path = remove_special_dirs_from_path (tmp1);
 		g_free (tmp1);
-		g_free (tmp2);
 
 		if (path_is_dir (path))
 			is_dir = TRUE;
@@ -445,13 +434,16 @@ initialize_data (poptContext pctx)
 		}
 
 		if (is_dir) {
-			dir_urls[n_dir_urls++] = get_uri_from_path (path);
+			dir_urls = g_list_prepend (dir_urls, get_uri_from_path (path));
 			g_free (path);
 		} else
-			file_urls[n_file_urls++] = path;
+			file_urls = g_list_prepend (file_urls, path);
 	}
 
-	if ((n_file_urls == 1) && (file_urls[0] != NULL)) {
+	n_file_urls = g_list_length (file_urls);
+	n_dir_urls = g_list_length (dir_urls);
+
+	if (n_file_urls == 1) {
 		view_single_image = TRUE;
 
 	} else if (n_file_urls > 1) {
@@ -459,13 +451,14 @@ initialize_data (poptContext pctx)
 		Catalog *catalog;
 		char    *catalog_path;
 		char    *catalog_name, *catalog_name_utf8;
+		GList   *scan;
 
 		catalog = catalog_new ();
 
 		catalog_name_utf8 = g_strconcat (_("Command Line"),
 						 CATALOG_EXT,
 						 NULL);
-		catalog_name = g_filename_from_utf8 (catalog_name_utf8, -1, 0, 0, 0);
+		catalog_name = gnome_vfs_escape_string (catalog_name_utf8);
 		catalog_path = get_catalog_full_path (catalog_name);
 		g_free (catalog_name);
 		g_free (catalog_name_utf8);
@@ -473,8 +466,10 @@ initialize_data (poptContext pctx)
 		catalog_set_path (catalog, catalog_path);
 		g_free (catalog_path);
 
-		for (i = 0; i < n_file_urls; i++) 
-			catalog_add_item (catalog, file_urls[i]);
+		for (scan = file_urls; scan; scan = scan->next)
+			catalog_add_item (catalog, scan->data);
+
+		catalog->sort_method = GTH_SORT_METHOD_MANUAL;
 
 		catalog_write_to_disk (catalog, NULL);
 		catalog_free (catalog);
@@ -487,16 +482,16 @@ initialize_data (poptContext pctx)
 
 
 /* Free application data. */
-static void 
+static void
 release_data (void)
 {
+#ifdef HAVE_GTKUNIQUE
 	if (gth_application != NULL)
-                bonobo_object_unref (gth_application);
+                g_object_unref (gth_application);
+#endif /* HAVE_GTKUNIQUE */
 
 	free_icon_pixbufs ();
 
-	g_object_unref (icon_theme);
-
 	if (monitor != NULL) {
 		g_object_unref (monitor);
 		monitor = NULL;
@@ -507,20 +502,24 @@ release_data (void)
 }
 
 
-static void 
-open_viewer_window (const char               *uri,
-		    gboolean                  use_factory,
-		    GNOME_GThumb_Application  app,
-		    CORBA_Environment        *env)
+static void
+open_viewer_window (const char *uri,
+		    gboolean    use_factory)
 {
 	GtkWidget *current_window;
 
+#ifdef HAVE_GTKUNIQUE
 	if (use_factory) {
+		char *command;
+
 		if (uri == NULL)
 			uri = "";
-		GNOME_GThumb_Application_open_viewer (app, uri, env);
+		command = g_strjoin ("|", "viewer", uri, NULL);
+    		gtk_unique_app_send_message (gth_application, GTK_UNIQUE_CUSTOM, command);
+		g_free (command);
 		return;
 	}
+#endif /* HAVE_GTKUNIQUE */
 
 	current_window = gth_viewer_new (uri);
 	gtk_widget_show (current_window);
@@ -529,21 +528,25 @@ open_viewer_window (const char               *uri,
 }
 
 
-static void 
-open_browser_window (const char               *uri,
-		     gboolean                  show_window,
-		     gboolean                  use_factory,
-		     GNOME_GThumb_Application  app,
-		     CORBA_Environment        *env)
+static void
+open_browser_window (const char *uri,
+		     gboolean    show_window,
+		     gboolean    use_factory)
 {
 	GtkWidget *current_window;
 
+#ifdef HAVE_GTKUNIQUE
 	if (use_factory) {
+		char *command;
+
 		if (uri == NULL)
 			uri = "";
-		GNOME_GThumb_Application_open_browser (app, uri, env);
+		command = g_strjoin ("|", "browser", uri, NULL);
+    		gtk_unique_app_send_message (gth_application, GTK_UNIQUE_CUSTOM, command);
+		g_free (command);
 		return;
 	}
+#endif /* HAVE_GTKUNIQUE */
 
 	current_window = gth_browser_new (uri);
 	if (show_window)
@@ -554,12 +557,10 @@ open_browser_window (const char               *uri,
 
 
 static void
-load_session (gboolean                  use_factory,
-	      GNOME_GThumb_Application  app,
-	      CORBA_Environment        *env)
+load_session (gboolean use_factory)
 {
 	int i, n;
-	
+
 	gnome_config_push_prefix (gnome_client_get_config_prefix (master_client));
 
 	n = gnome_config_get_int ("Session/locations");
@@ -571,9 +572,9 @@ load_session (gboolean                  use_factory,
 		location = gnome_config_get_string (key);
 
 		if (uri_scheme_is_file (location) && path_is_file (location))
-			open_viewer_window (location, use_factory, app, env);
+			open_viewer_window (location, use_factory);
 		else
-			open_browser_window (location, TRUE, use_factory, app, env);
+			open_browser_window (location, TRUE, use_factory);
 
 		g_free (location);
 		g_free (key);
@@ -583,119 +584,205 @@ load_session (gboolean                  use_factory,
 }
 
 
-/* Create the windows. */
-static void 
-prepare_app (void)
+static void
+show_grabbing_focus (GtkWidget *new_window)
 {
-	CORBA_Object              factory;
-	gboolean                  use_factory = FALSE;
-	CORBA_Environment         env;
-	GNOME_GThumb_Application  app;
-	int                       i;
-	
-	factory = bonobo_activation_activate_from_id ("OAFIID:GNOME_GThumb_Application_Factory",
-                                                      Bonobo_ACTIVATION_FLAG_EXISTING_ONLY,
-                                                      NULL, NULL);
+	const char *startup_id = NULL;
+	guint32     timestamp = 0;
 
-	if (factory != NULL) {
-		use_factory = TRUE;
-		CORBA_exception_init (&env);
-		app = bonobo_activation_activate_from_id ("OAFIID:GNOME_GThumb_Application", 0, NULL, &env);
+	gtk_widget_realize (new_window);
+
+	startup_id = g_getenv ("DESKTOP_STARTUP_ID");
+	if (startup_id != NULL) {
+		char *startup_id_str = g_strdup (startup_id);
+		char *ts;
+
+		ts = g_strrstr (startup_id_str, "_TIME");
+		if (ts != NULL) {
+			ts = ts + 5;
+			errno = 0;
+			timestamp = strtoul (ts, NULL, 0);
+			if ((errno == EINVAL) || (errno == ERANGE))
+				timestamp = 0;
+		}
+
+		g_free (startup_id_str);
 	}
 
+	if (timestamp == 0)
+		timestamp = gdk_x11_get_server_time (new_window->window);
+	gdk_x11_window_set_user_time (new_window->window, timestamp);
+
+	gtk_window_present (GTK_WINDOW (new_window));
+}
+
+
+#ifdef HAVE_GTKUNIQUE
+static GtkUniqueResponse
+application_message_cb (GtkUniqueApp     *app,
+                        GtkUniqueCommand  command,
+                        const gchar      *command_data,
+                        const gchar      *startup_id,
+                        GdkScreen        *screen,
+                        guint             workspace,
+                        gpointer          user_data)
+{
+	char **tokens;
+	char *sub_command;
+	char *uri;
+
+	if (command != GTK_UNIQUE_CUSTOM)
+		return GTK_UNIQUE_RESPONSE_ABORT;
+
+	tokens = g_strsplit (command_data, "|", 2);
+	sub_command = tokens[0];
+	uri = tokens[1];
+
+	if ((uri != NULL) && (*uri == '\0'))
+		uri = NULL;
+
+      	if (strcmp (sub_command, "browser") == 0)
+        	show_grabbing_focus (gth_browser_new (uri));
+
+	else if (strcmp (sub_command, "viewer") == 0)
+        	show_grabbing_focus (gth_viewer_new (uri));
+
+      	else if (strcmp (sub_command, "load_image") == 0) {
+		if (UseViewer) {
+			GtkWidget *viewer = gth_viewer_get_current_viewer ();
+			if (viewer == NULL)
+				show_grabbing_focus (gth_viewer_new (uri));
+			else {
+				gth_viewer_load (GTH_VIEWER (viewer), uri);
+				show_grabbing_focus (viewer);
+			}
+		} else {
+			GtkWidget *browser = gth_browser_get_current_browser ();
+			if (browser == NULL)
+				show_grabbing_focus (gth_browser_new (uri));
+			else {
+				gth_browser_load_uri (GTH_BROWSER (browser), uri);
+				show_grabbing_focus (browser);
+			}
+		}
+      	}
+      	else if (strcmp (sub_command, "import") == 0)
+        	gth_browser_activate_action_file_camera_import (NULL, NULL);
+
+	return GTK_UNIQUE_RESPONSE_OK;
+}
+#endif /* HAVE_GTKUNIQUE */
+
+
+/* Create the windows. */
+static void
+prepare_app (void)
+{
+	gboolean  use_factory = FALSE;
+	GList    *scan;
+
+#ifdef HAVE_GTKUNIQUE
+	gth_application = gtk_unique_app_new ("org.gnome.GThumb");
+	use_factory = gtk_unique_app_is_running (gth_application);
+
+	if (! use_factory)
+		g_signal_connect (gth_application, "message",
+                		  G_CALLBACK (application_message_cb),
+                          	  NULL);
+#endif /* HAVE_GTKUNIQUE */
+
 	if (session_is_restored ()) {
-		load_session (use_factory, app, &env);
+		load_session (use_factory);
 		return;
 	}
 
 	if (ImportPhotos) {
+#ifdef HAVE_GTKUNIQUE
 		if (use_factory)
-			GNOME_GThumb_Application_import_photos (app, &env);
-		else {
+		 	gtk_unique_app_send_message (gth_application, GTK_UNIQUE_CUSTOM, "import");
+		else
+#endif /* HAVE_GTKUNIQUE */
 			gth_browser_activate_action_file_camera_import (NULL, NULL);
-			/*
-			preferences_set_startup_location (NULL);
-			open_browser_window (NULL, FALSE, use_factory, app, &env);
-			*/
-		}
 
-	} else if (! view_comline_catalog 
-	    && (n_dir_urls == 0) 
+	} else if (! view_comline_catalog
+	    && (n_dir_urls == 0)
 	    && (n_file_urls == 0)) {
-		open_browser_window (NULL, TRUE, use_factory, app, &env);
+		open_browser_window (NULL, TRUE, use_factory);
 
 	} else if (view_single_image) {
-		if (use_factory && eel_gconf_get_boolean (PREF_SINGLE_WINDOW, FALSE))
-			GNOME_GThumb_Application_load_image (app, file_urls[0], &env);
-		else
-			open_viewer_window (file_urls[0], use_factory, app, &env);
+		if (use_factory && eel_gconf_get_boolean (PREF_SINGLE_WINDOW, FALSE)) {
+#ifdef HAVE_GTKUNIQUE
+			char *command = g_strjoin ("|", "load_image", file_urls->data, NULL);
+    			gtk_unique_app_send_message (gth_application, GTK_UNIQUE_CUSTOM, command);
+			g_free (command);
+#endif /* HAVE_GTKUNIQUE */
+		} else {
+			if (UseViewer)
+				open_viewer_window (file_urls->data, use_factory);
+			else
+				open_browser_window (file_urls->data, TRUE, use_factory);
+		}
 
 	} else if (view_comline_catalog) {
-		char *catalog_uri;
-		char *catalog_path;
+		char *tmp;
 		char *catalog_name;
+		char *catalog_path;
+		char *catalog_uri;
 
 		ViewFirstImage = TRUE;
 		HideSidebar = TRUE;
 
-		catalog_name = g_strconcat (_("Command Line"), CATALOG_EXT, NULL);
+		tmp = g_strconcat (_("Command Line"), CATALOG_EXT, NULL);
+		catalog_name = gnome_vfs_escape_string (tmp);
+		g_free (tmp);
+
 		catalog_path = get_catalog_full_path (catalog_name);
 		catalog_uri = g_strconcat ("catalog://", catalog_path, NULL);
 
-		open_browser_window (catalog_uri, TRUE, use_factory, app, &env);
+		open_browser_window (catalog_uri, TRUE, use_factory);
 
 		g_free (catalog_name);
 		g_free (catalog_path);
 		g_free (catalog_uri);
 	}
 
-	for (i = 0; i < n_dir_urls; i++) {
+	for (scan = dir_urls; scan; scan = scan->next) {
 		/* Go to the specified directory. */
-		open_browser_window (dir_urls[i], TRUE, use_factory, app, &env);
+		open_browser_window (scan->data, TRUE, use_factory);
 	}
 
-	/* Free urls. */
-
-	if (n_file_urls > 0) {
-		for (i = 0; i < n_file_urls; i++)
-			g_free (file_urls[i]);
-		g_free (file_urls);
-	}
-
-	if (n_dir_urls > 0) {
-		for (i = 0; i < n_dir_urls; i++)
-			g_free (dir_urls[i]);
-		g_free (dir_urls);
-	}
+	path_list_free (file_urls);
+	path_list_free (dir_urls);
 
 	/**/
 
 	if (use_factory) {
-		bonobo_object_release_unref (app, &env);
-		CORBA_exception_free (&env);
 		gdk_notify_startup_complete ();
 		exit (0);
-	} else
-		gth_application = gth_application_new (gdk_screen_get_default ());
+	}
 }
 
 
-int 
-main (int   argc, 
+int
+main (int   argc,
       char *argv[])
 {
 	GnomeProgram *program;
-	GValue        value = { 0 };
-	poptContext   pctx;
+	char         *description;
 
 	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
+	description = g_strdup_printf ("- %s", _("View and organize your images"));
+        context = g_option_context_new (description);
+        g_free (description);
+        g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+
 	program = gnome_program_init ("gthumb", VERSION,
-				      LIBGNOMEUI_MODULE, 
+				      LIBGNOMEUI_MODULE,
 				      argc, argv,
-				      GNOME_PARAM_POPT_TABLE, options,
+				      GNOME_PARAM_GOPTION_CONTEXT, context,
 				      GNOME_PARAM_HUMAN_READABLE_NAME, _("gThumb"),
 				      GNOME_PARAM_APP_PREFIX, GTHUMB_PREFIX,
 				      GNOME_PARAM_APP_SYSCONFDIR, GTHUMB_SYSCONFDIR,
@@ -708,15 +795,10 @@ main (int   argc,
 		gdk_threads_init ();
 	}
 
-	g_object_get_property (G_OBJECT (program),
-			       GNOME_PARAM_POPT_CONTEXT,
-			       g_value_init (&value, G_TYPE_POINTER));
-
-	pctx = g_value_get_pointer (&value);
+	gnome_authentication_manager_init ();
 	glade_gnome_init ();
 	gthumb_init ();
-	initialize_data (pctx);
-	poptFreeContext (pctx);
+	initialize_data ();
 	prepare_app ();
 
 	gtk_main ();
@@ -729,14 +811,14 @@ main (int   argc,
 /**/
 
 
-void 
+void
 all_windows_update_catalog_list (void)
 {
 	gth_monitor_notify_reload_catalogs (monitor);
 }
 
 
-void 
+void
 all_windows_notify_update_bookmarks (void)
 {
 	gth_monitor_notify_update_bookmarks (monitor);
@@ -747,9 +829,9 @@ void
 all_windows_notify_cat_files_created (const char *catalog_path,
 				      GList      *list)
 {
-	gth_monitor_notify_update_cat_files (monitor, 
-					     catalog_path, 
-					     GTH_MONITOR_EVENT_CREATED, 
+	gth_monitor_notify_update_cat_files (monitor,
+					     catalog_path,
+					     GTH_MONITOR_EVENT_CREATED,
 					     list);
 }
 
@@ -758,35 +840,35 @@ void
 all_windows_notify_cat_files_deleted (const char *catalog_path,
 				      GList      *list)
 {
-	gth_monitor_notify_update_cat_files (monitor, 
-					     catalog_path, 
-					     GTH_MONITOR_EVENT_DELETED, 
+	gth_monitor_notify_update_cat_files (monitor,
+					     catalog_path,
+					     GTH_MONITOR_EVENT_DELETED,
 					     list);
 }
 
 
-void 
+void
 all_windows_notify_files_created (GList *list)
 {
 	gth_monitor_notify_update_files (monitor, GTH_MONITOR_EVENT_CREATED, list);
 }
 
 
-void 
+void
 all_windows_notify_files_deleted (GList *list)
 {
 	gth_monitor_notify_update_files (monitor, GTH_MONITOR_EVENT_DELETED, list);
 }
 
 
-void 
+void
 all_windows_notify_files_changed (GList *list)
 {
 	gth_monitor_notify_update_files (monitor, GTH_MONITOR_EVENT_CHANGED, list);
 }
 
 
-void 
+void
 all_windows_notify_file_rename (const gchar *old_name,
 				const gchar *new_name)
 {
@@ -806,14 +888,14 @@ all_windows_notify_files_rename (GList *old_names,
 
 		gth_monitor_notify_file_renamed (monitor, old_name, new_name);
 		all_windows_notify_file_rename (old_name, new_name);
-		
+
 		o_scan = o_scan->next;
 		n_scan = n_scan->next;
 	}
 }
 
 
-void 
+void
 all_windows_notify_directory_rename (const gchar *old_name,
 				     const gchar *new_name)
 {
@@ -821,14 +903,14 @@ all_windows_notify_directory_rename (const gchar *old_name,
 }
 
 
-void 
+void
 all_windows_notify_directory_delete (const char *path)
 {
 	gth_monitor_notify_update_directory (monitor, path, GTH_MONITOR_EVENT_DELETED);
 }
 
 
-void 
+void
 all_windows_notify_directory_new (const char *path)
 {
 	gth_monitor_notify_update_directory (monitor, path, GTH_MONITOR_EVENT_CREATED);
@@ -864,7 +946,7 @@ all_windows_notify_catalog_reordered (const gchar *path)
 }
 
 
-void 
+void
 all_windows_notify_update_metadata (const gchar *filename)
 {
 	gth_monitor_notify_update_metadata (monitor, filename);
@@ -878,17 +960,19 @@ all_windows_notify_update_icon_theme (void)
 }
 
 
-void
+gboolean
 all_windows_remove_monitor (void)
 {
 	gth_monitor_pause (monitor);
+	return FALSE;
 }
 
 
-void
+gboolean
 all_windows_add_monitor (void)
 {
 	gth_monitor_resume (monitor);
+	return FALSE;
 }
 
 
@@ -908,33 +992,29 @@ get_fs_icon (IconName icon_name,
 	icon_pixbuf[icon_name] = create_pixbuf (icon_theme,
 						icon_mime_name[icon_name],
 						icon_size);
-	
+
 	return icon_pixbuf[icon_name];
 	*/
 
-	if (icon_pixbuf[icon_name] == NULL) {
-		const GnomeIconData *icon_data;
-		int                  base_size;
-		char                *icon_path;
-		
-		icon_path = gnome_icon_theme_lookup_icon (icon_theme,
-							  icon_mime_name[icon_name],
-							  icon_size,
-							  &icon_data,
-							  &base_size);
+        if (icon_pixbuf[icon_name] == NULL) {
+		GtkIconInfo         *icon_info = NULL;
 
-		if (icon_path == NULL) {
-			icon_pixbuf[icon_name] = gdk_pixbuf_new_from_inline (-1, 
-									     dir_16_rgba, 
-									     FALSE, 
-									     NULL);
-			scale = FALSE;
-		} else {
-			icon_pixbuf[icon_name] = gdk_pixbuf_new_from_file (icon_path, 
-									   NULL);
-			g_free (icon_path);
-		}
-	}
+		icon_info = gtk_icon_theme_lookup_icon (icon_theme,
+							icon_mime_name[icon_name],
+							icon_size,
+							0);
+
+		if (icon_info == NULL) {
+                	icon_pixbuf[icon_name] = gdk_pixbuf_new_from_inline (-1,
+                                                                             dir_16_rgba,
+                                                                             FALSE,
+                                                                             NULL);
+                        scale = FALSE;
+                } else {
+			icon_pixbuf[icon_name] = gtk_icon_info_load_icon (icon_info, NULL);
+			gtk_icon_info_free (icon_info);
+                }
+        }
 
 	/* Scale keeping aspect ratio. */
 
@@ -945,10 +1025,10 @@ get_fs_icon (IconName icon_name,
 
 	if (icon_pixbuf[icon_name] != NULL) {
 		int w, h;
-		
+
 		w = gdk_pixbuf_get_width (icon_pixbuf[icon_name]);
 		h = gdk_pixbuf_get_height (icon_pixbuf[icon_name]);
-		if (scale_keepping_ratio (&w, &h, icon_size, icon_size)) 
+		if (scale_keepping_ratio (&w, &h, icon_size, icon_size))
 			pixbuf = gdk_pixbuf_scale_simple (icon_pixbuf[icon_name],
 							  w,
 							  h,
@@ -1022,15 +1102,15 @@ get_stock_id_for_uri (const char *uri)
 {
 	const char *stock_id;
 
-	if (strcmp (uri, g_get_home_dir ()) == 0) 
+	if (strcmp (uri, g_get_home_dir ()) == 0)
 		stock_id = GTK_STOCK_HOME;
-	else if (strcmp (uri, get_home_uri ()) == 0) 
+	else if (strcmp (uri, get_home_uri ()) == 0)
 		stock_id = GTK_STOCK_HOME;
 	else if (folder_is_film (uri))
 		stock_id = GTHUMB_STOCK_FILM;
-	else if (uri_scheme_is_catalog (uri)) 
+	else if (uri_scheme_is_catalog (uri))
 		stock_id = GTHUMB_STOCK_CATALOG;
-	else if (uri_scheme_is_search (uri))	
+	else if (uri_scheme_is_search (uri))
 		stock_id = GTHUMB_STOCK_SEARCH;
 	else
 		stock_id = GTK_STOCK_OPEN;
@@ -1048,23 +1128,27 @@ get_icon_for_uri (GtkWidget  *widget,
 
 	menu_size = get_folder_pixbuf_size_for_list (widget);
 
-	if (strcmp (uri, g_get_home_dir ()) == 0) 
+	if (strcmp (uri, g_get_home_dir ()) == 0)
 		return get_fs_icon (ICON_NAME_HOME, menu_size);
-	else if (strcmp (uri, get_home_uri ()) == 0) 
+	else if (strcmp (uri, get_home_uri ()) == 0)
 		return get_fs_icon (ICON_NAME_HOME, menu_size);
 
-	if ((strcmp (uri, "file://") == 0) || (strcmp (uri, "/") == 0)) 
+	if ((strcmp (uri, "file://") == 0) || (strcmp (uri, "/") == 0))
 		return get_fs_icon (ICON_NAME_HARDDISK, menu_size);
 
 	if (folder_is_film (uri))
 		stock_id = GTHUMB_STOCK_FILM;
-	else if (uri_scheme_is_catalog (uri)) 
-		stock_id = GTHUMB_STOCK_CATALOG;
-	else if (uri_scheme_is_search (uri))	
+	else if (uri_scheme_is_catalog (uri)) {
+		if (file_extension_is (uri, CATALOG_EXT))
+			stock_id = GTHUMB_STOCK_CATALOG;
+		else
+			stock_id = GTHUMB_STOCK_LIBRARY;
+	}
+	else if (uri_scheme_is_search (uri))
 		stock_id = GTHUMB_STOCK_SEARCH;
 
 	if (stock_id != NULL)
-		return gtk_widget_render_icon (widget, stock_id, GTK_ICON_SIZE_MENU, NULL); 
+		return gtk_widget_render_icon (widget, stock_id, GTK_ICON_SIZE_MENU, NULL);
 	else
 		return get_fs_icon (ICON_NAME_DIRECTORY, menu_size);
 }
